#ifndef __BThread_h__
#define __BThread_h__

#include "AutoSocket.h"
#include "EngineConfiguration.h"
#include "IBThread.h"
#include "SyncRequest.h"
#include "Vector.h"
#include <arpa/inet.h>

class BThread : public IBThread
{
public:
    BThread() : _socket( inet_addr( EngineConfiguration::LOOPBACK_IP ), EngineConfiguration::BTHREAD_MANAGEMENT_PORT )
    {
    }

    void bSync( const Vector<Event> &requested,
                const Vector<Event> &watched,
                const Vector<Event> &blocked,
                String label = "" )
    {
        AtomicPropositions noPropositions;

        sendSyncRequest( requested, watched, blocked, noPropositions, label );
        waitForSyncResponse();
    }

    void bSync( const Vector<Event> &requested,
                const Vector<Event> &watched,
                const Vector<Event> &blocked,
                const AtomicPropositions &atomicPropositions,
                String label = "" )
    {
        sendSyncRequest( requested, watched, blocked, atomicPropositions, label );
        waitForSyncResponse();
    }

    Event lastEvent() const
    {
        return _lastEvent;
    }

    void done()
    {
        _socket->closeSocket();
    }

private:
    AutoSocket _socket;
    Event _lastEvent;

    void waitForSyncResponse()
    {
        HeapData receivedData;
        _socket->receive( receivedData );
        _lastEvent.deserialize( ConstSimpleData( receivedData.data(), receivedData.size() ) );
    }

    void sendSyncRequest( const Vector<Event> &requested,
                          const Vector<Event> &watched,
                          const Vector<Event> &blocked,
                          const AtomicPropositions &atomicPropositions,
                          String &label )
    {
        SyncRequest syncRequest( requested, watched, blocked, atomicPropositions, label );

        HeapData serialized;
        syncRequest.serialize( serialized );

        _socket->send( serialized );
    }
};

#endif // __BThread_h__

//
// Local Variables:
// compile-command: "make -C .. "
// tags-file-name: "../TAGS"
// c-basic-offset: 4
// End:
//
